#-------------------------------------------------------------------------------
# Copyright 2017 Visual Components Ltd. All rights reserved.
#-------------------------------------------------------------------------------
from vcCommand import *
from vcHelpers.Selection import *
import os.path, vcMatrix, vcVector
## simple example to start the the topology continuous curve picking -command
## when user clicks 'done' after picking the topology element, the command creates frames with 10mm step along the curvature
## every time the command is called the previously created frames are deleted
## after creating Frames press the [Create Surface] button to create Surface. 

app = getApplication()

def picked(cmd):
  global topo_pick, comp

  comp = None
  name_index = 0
  # Delete all frames from the picked component(s)
  for t in topo_pick.TargetCurves:
    node = t[0]
    comp = node.Component
    for i in comp.RootFeature.Children:
      if i.Type == VC_FRAME and 'Pos_' in i.Name:
        i.delete()
    for i in comp.RootFeature.Children:
      if 'Frames Holder' in i.Name:
        i.delete()    
  # Use topology data of the picked curves for creating frames along the curves
  if len(comp.RootFeature.Children) == 0:
   transform_node = comp.RootFeature.createFeature(VC_TRANSFORM,"Frames Holder") 
  
  for i in comp.RootFeature.Children:
   if i.Type == VC_TRANSFORM and 'Frames' in i.Name:
    transform_node = i
   else:
    transform_node = comp.RootFeature.createFeature(VC_TRANSFORM,"Frames Holder")     
  
  for t in topo_pick.TargetCurves: 
   node = t[0]
   g_set = t[1]
   curve_index = t[2]
   curve_direction = t[3]
   comp = node.Component
   topo = g_set.Topology
   le = topo.getCurveLength(curve_index)
   ### still not implemented ... getCurveEnds(curve_index) ... ###
   #vec_start,vec_end,vec_start_real,vec_end_real = topo.getCurveEnds(curve_index)
   #print vec_start,vec_end,vec_start_real,vec_end_real
   a = 0
   while a < le:
     pos = topo.getCurvePosition(curve_index, a) # returns a matrix position at a certain length on the curve
     a += getProperty('Frame Distance(mm)').Value
     name_index += 1
     frame = transform_node.createFeature(VC_FRAME, 'Pos_%i' % name_index)
     frame.PositionMatrix = pos
     frame.rebuild()
  topo_pick.OnTargetSet = None

def nowPick(arg):
  global topo_pick

  topo_pick = app.findCommand('interactiveTopologyPick')
  topo_pick.SnapOnCurveLoop = True
  topo_pick.ContinousMode = True #allows user to pick a chain of multiple curves
  topo_pick.FaceDisplayMode = VC_FACEDISPLAY_FACE
  topo_pick.OnTargetSet = picked
  topo_pick.execute()
  print 'step-1: Pick curve and then press [Enter] in keyboard'
  print 'step-2: Press [Create Surface] to create surface along the Frames'  

def createCurve(arg):
  global comp, pos_previous_WPR, geoSet

  geoFeat = comp.RootFeature.createFeature(VC_GEOMETRY, "Surface_Geometry") 
  geoSet = geoFeat.Geometry.createGeometrySet(VC_TRIANGLESET)
  child = comp.findFeature("Frames Holder")
  pos_previous_WPR = 0
  counter = 0

  geoSet.addPoint(child.Children[0].PositionMatrix.P)
  for i in child.Children:
    if i.Type == VC_FRAME:
      vec1 = i.PositionMatrix.P
      if notOnSameLine(i):  
        geoSet.addPoint(vec1)
      pos_previous_WPR = i.PositionMatrix.WPR
  geoSet.addPoint(child.Children[-1].PositionMatrix.P)
  
  for i in range(1,geoSet.PointCount-1):
    geoSet.addTriangle(0,i,i+1)
  
  comp.rebuild()  
  frame_holder = comp.findFeature("Frames Holder")
  frame_holder.delete()     

def notOnSameLine(pos_current):
  global pos_previous_WPR

  if pos_previous_WPR:
    pos_current_WPR = pos_current.PositionMatrix.WPR
    if pos_current_WPR ==  pos_previous_WPR:
      return False
    else: 
      return True                  
### function reverse() still not implemented ###
'''
def invertSurface(arg):
  global geoSet, comp
  
  print geoSet
  geoSet.reverse()
'''

def first_state():
  global prop1, prop2, prop3, prop4

  i = 10
  prop1 = createProperty(VC_BUTTON,'Pick Curve')
  prop1.IsVisible = True
  prop1.OnChanged = nowPick
  prop1.Group = i
  i += 10
  prop2 = createProperty(VC_BUTTON,'Create Surface')
  prop2.IsVisible = True
  prop2.OnChanged = createCurve
  prop2.Group = i
  i += 10
  prop3 = createProperty(VC_REAL,'Frame Distance(mm)')
  prop3.Value = 10
  prop3.IsVisible = True
  prop3.Group = i
  i += 10
  '''prop4 = createProperty(VC_BUTTON,'Invert Surface')
  prop4.IsVisible = True
  prop4.OnChanged = invertSurface
  prop4.Group = i'''

  executeInActionPanel()


addState( first_state )


    